# by DCL manual
=begin
==NumRu::AdvancedDCL::Tone
==={W[ɂ
ToneW[̓J[g[(mN)\邽߂̃W[łB
ʂ킯UWnłȂ̂, ׂĂ̐KϊɑΉĂ. 

̃pbP[Wł, ʂ킯Ȃ l̂̂W̊OgȂǂ͕`Ȃ. 

Tone ɂă[U[gpTu[`ƂĂ͈ȉ̂悤Ȃ̂. 
* 惁\bh
  * draw DDDg[ɂ2l}̂ʂ킯Ȃ.
  * imdraw DDDC[W`@\ɂ2l}̂ʂ킯Ȃ.
  * fill DDDp`̈ʂԂ. 
  * vfill DDDVWnőp`̈ʂԂ.                             
  * rfill DDD  RWnőp`̈ʂԂ.                           * ʂ킯郌xɊւ郁\bh
  * gtla DDDŏlEőlEݒl ʂ킯郌x.
  * gtlb DDDiq_l^zƂݒl ʂ킯郌x𐶐.
  * stlv DDDʂ킯郌xƃp^[1xƎw肷.
  * stln DDDʂ킯郌xƃp^[zŕxw肷.
  * itlv DDDʂ킯郌xƃp^[ ׂĖɂ.
* ̑̃\bh
  * get, set DDDp^Q/ύX.

iq_WɊւẮA惁\bhɊiq_(WlƂ)^ƎIɐݒ肵Ă.

2f[^Ƃɂʂ킯(quick look)ƂƂɂ, P Tone.draw ĂԂł悢. ̂Ƃ, ݐݒ肳ĂEChEς Ԋůiq_ݒ肵, ̗̈Ɏΐ̃nb`. 

ʂ킯郌xl̐ݒ Contour ƓlɂĂȂƂ, Tone ̃p^ ltone true Ƃ Tone.draw Ăׂ, Contour Iɐ邫ݒl (ɂ Contour ƓASYɂƂÂăxlݒ肵)ʂ킯. , Contour.gcla ɑΉ Tone.gtla, Contour.gclb ɑΉ Tone.gtlb Ăׂ R^[xƂʂ킯郌x̒lɐݒ肳. 

iq_sԊȕꍇ, iq_̍Wl`惁\bḧƂĎw肷΂悢.

ɏqׂ悤, ʂ킯郌x炩ߎw肵Ƃ, Tone.drawĂԑO Tone.gtla ܂ Tone.gtlb Ăׂ΂悢. ̏ꍇ, Ԋũg[xȂ. , sԊũg[xƃp^[w肵ꍇ, ̃g[xƃp^[ǉꍇ, Tone.stlv ܂ Tone.stln Ăׂ΂悢. (ׂẴx𖳌ɂɂ Tone.itlv Ăׂ΂悢. ) Ȃ,ʂ킯͎w肵x̏ɂȂ. 

̃W[pĂg[v~eBu o͑u̔\͂ɉhard fillsoft fillؑւ邱Ƃł. Tone Ǘp^ lsoftf  true ̂Ƃsoft fillȂ; false ̂Ƃ hard fillȂ (l true). , hard fillw肵Ă̔\͂Ȃꍇ soft fillƂȂ. 

܂, Fpׂh(3999ł悤ȃg[p^[ԍ) ɂh킯ȂƂ, FԍhbgȂǂ̃p^[ԍƂēǂ݂邱Ƃł. Tone ̊Ǘp^ lcl2tn  true ̂Ƃ͓ǂ݂; false ̂Ƃ͓ǂ݂Ȃ (l false). , Fph킯łȂ悤ȑuł, lcl2tn  false Ɛݒ肳ĂĂ, p^[ԍɂǂ݂ȂƂ. 

łɕ`ꂽ}`̏soft fillȂĂ, g[p^[㏑邾, ܂łɕ`ꂽ}`(ׂhȂ)Ă܂Ƃ͂Ȃ. Ȃ, o͑uɂĂhard fillɂʂ킯ɂĐɕ`ꂽ}`Ă܂Ƃ̂,Wⓙ}ƈꏏUEPACKpƂ `Ԃɒӂ邱.
 
===p^
p^ p^̌^ ΉDCLpbP[W
:icolor1 (Fixnum) <UEPACK>
   ltone  true ̂ƂpFԍ̉l(l15).
:icolor2 (Fixnum) <UEPACK>
   ltone  true ̂ƂpFԍ̏l(l94).
:ipat (Fixnum) <UEPACK>
   ltone  false ̂Ƃ, ϐ rlev ȉ̗̈ʂg[p^[(l1201:Eオ̎ΐ).
:nlev (Fixnum) <UEPACK>
   g[xIɐƂ,̓ϐQƂĖ nlev {̃R^[x𐶐(l12).
:icycle (Fixnum) <UEPACK>
   minor line  major line ̊Ԃɉ{w肷. Ȃ킿 icycle ŎQƂl1{̊ major line , c minor line (l2).
:ltone (true or false) <UEPACK>
   ʂ킯g[x[U[w肵ȂƂ, Tone.gtlb găg[xƃg[p^[𐶐邩ǂw肷. true Ȃ Tone.gtlb ; false Ȃ rlev (L)ȉʂ悤Ȑݒ(l false).
:rlev (Float) <UEPACK>
   ltone  false ̂Ƃ, ʂ킯̈̋El(l0.0).Ŏw肵lȉ̗̈ʂ.
:lsoftf (true or false) <SGPACK>
   g[v~eBuœhԂȂƂ soft fill Ȃ hard fill Ȃw肷. true ̂Ƃ soft fill Ȃ; false ̂Ƃ hard fill Ȃ(l false). , hard fill w肵Ă̔\͂Ȃꍇ soft fillƂȂ.   
:lcl2tn (true or false) <SGPACK>
   Fpׂhɂh킯ȂƂ, FԍhbgȂǂ̃p^[ԍƂēǂ݂邩ǂw肷. true ̂Ƃ͓ǂ݂; false ̂Ƃ͓ǂ݂Ȃ(l false)., Fph킯łȂ悤ȑuł, false Ɛݒ肳ĂĂp^[ԍɂǂ݂ȂƂ.
:tnblen (Float) <SGPACK>
   g[v~eBuŃhbgp^[ɂ soft fill Ȃꍇ,tBGATuv~eBủʃ[`p_̏WƂĕ\.̂Ƃ_̃C^Cv̓rbgp^[Őݒ肳邪,1rbg RC łǂꂾ̒ɑ邩w肷(l0.001).
:irmode (Fixnum) <SGPACK>
   g[v~eBuŎw肳ꂽ̈悪NbsOEɂ鎞, Ěw肷p^. 0 ̂Ƃv; 1 ̂Ƃv. DCL manual 1.7.5ߎQƂ̂.(l0).
:tonepat (Fixnum) <UEPACK, SGPACK>
   ltone  true ̂Ƃpp^[ԍ(l999, ׂh).

܂Ãp^̓\bhƂėp邱ƂłBȂ킿ATone.nlev Ńp^ nlev ̎QƂATone.nlev= Ńp^ nlev ̕ύXłB

=end

module NumRu
  module AdvancedDCL
    module Tone
      def param_init
	@param = Parameter.new(Common.param)
	
#--------------------- uepack parameter -------------------------
	@param.init("icolor1", Fixnum, "ueiset", "ueiget", 0, 0)
	@param.init("icolor2", Fixnum, "ueiset", "ueiget", 0, 0)
	@param.init("ipat", Fixnum, "ueiset", "ueiget", 0, 0)
	@param.init("nlev", Fixnum, "ueiset", "ueiget", 0, 0)
	@param.init("icycle", Fixnum, "ueiset", "ueiget", 0, 0)
	@param.init("lbound", TrueClass, "uelset", "uelget", 0, 0)
	@param.init("ltone", TrueClass, "uelset", "uelget", 0, 0)
	@param.init("rlev", Float, "uerset", "uerget", 0, 0)

#--------------------- sgpack parameter -------------------------
	@param.init("lsoftf", TrueClass, "sglset", "sglget", 0, 0)
	@param.init("lcl2tn", TrueClass, "sglset", "sglget", 0, 0)
	@param.init("tnblen", Float, "sgrset", "sgrget", 0, 0)
	@param.init("irmode", Fixnum, "sgiset", "sgiget", 0, 0)
	
#--------------------- common parameter -------------------------
	@param.init("tonepat", Fixnum, "tonepat", "tonepatget", 3, 3)
	
	@temp_param = @param.dup
	@backup = @param.dup
      
      end
      
      private :param_init

#------------------- method for parameters----------------------
    
      def set(key, val)
	@param[key] = val
      end

# by kawanabe
=begin
---Tone.set(key, val)
    p^̐ݒsB

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB
    gp
        Tone.set("nlev", 10)      # nlev̒l 10 ɐݒ肷

=end

      def get(key)
	@param[key] = val
      end

# by kawanabe
=begin
---Tone.get(key)
    p^QƂB

    
    * key:p^(String)
    gp
        Tone.gox("nlev")      # nlev ̒lQƂ

=end

      def gtla(max, min, dx)
	@param.sync
	
	DCL.uegtla(max, min, dx)
      end

# by DCL manual
=begin
---Tone.gtla(max, min, dx)
    Tone.draw Ŏgg[xl ŏlEől ݒl܂͂悻̃xŐݒ肷.

    
    * max, min (Float) : R^[x̍ŏlƍől. K̂悢lłȂĂ悢.
    * dx (Float) : dx0̂Ƃ dxݕƂ. dx=0̂Ƃp^ nlev QƂ,  nlev { R^[x𐶐. dx0̂Ƃ (dx.abs.to_i) {̃R^[x𐶐. 

    l
    (1) ̃\bh UEGTLA Ăł.
    (2) ̃[` Contour.gcla (DCL manual 8.2.3ߎQ) ƓASYɂƂÂăxl𐶐Ă. ASY𓯈ɂ邽߁AContour ŗpp^, nlev, icycle Ɠ̂, Tone łpӂĂ. Contour ł̒lςƂ, Tone łύXĂȂ΂ȂȂACommon nlev, icycleς邱ƂŗύX邱Ƃ\łB

=end

      def gtlb(z, dx)
	@param.sync
	
	DCL.uegtlb(z, z.shape[0], z.shape[0], z.shape[1], dx)
      end

# by DCL manual
=begin
---Tone.gtlb(z, dx)
    Tone.draw ŎgR^[xliq_l^zƂݒl܂͂悻̃xŐݒ肷.

    
    * z (NArray) : 2z 
    * dx (Float) : dx0̂Ƃ dxݕƂ. dx=0̂Ƃp^ nlev QƂ,  nlev { R^[x𐶐. dx0̂Ƃ (dx.abs.to_i) {̃R^[x𐶐. 

    l
    (1) ̃\bh UEGTLB Ăł.
    (2) ̃[` z ̊iq_l̍ŏlEől`FbN,  Tone.gtla Ăł. 

=end

      def stlv(tlev1, tlev2, ipat)
	@param.sync
	
	DCL.uestlv(tlev1, tlev2, ipat)
      end

# by DCL manual
=begin
---Tone.stlv(zlev, index, itype, clv, hl)
    R^[Cׂ̑Ďw肵1{̃R^[xݒ肷.

    
    * zlev (Float) : R^[x̒l.
    * index (Fixnum) : R^[C̃CCfNX.
    * itype (Fixnum) : R^[C̃C^Cv.
    * clv (String) : R^[Cɂ郉x.
    * hl (Float) : x̍(PʂVWn). hl=0 ̂ƂxȂ.

    l
    (1) ̃\bh UDSTLV Ăł.
    (2) Tone.gtla, Tone.gtlb ͓ł̃\bhĂł. ⏕ꂽƂȂ Tone.gtla, Tone.gtlb Ă񂾌ł̃\bhĂׂ, ׂR^[x1{ǉł. 
    (3) x(clv)̒ 8 ȉłȂ΂ȂȂ. 
    (4) ݐݒ肳ĂR^[x̑ 
         Tone.levelget
    ɂĎQƂł. 
    
=end

      def stln(tlevn, ipatn)
	@param.sync
	
	if(!(tlevn.length == ipatn.length + 1))
	  raise ArgumentError, "length mismatch between tone level & tone number"
	end
	DCL.uestln(tlevn, ipatn, ipatn.length)
      end

# by DCL manual
=begin
---Tone.stln(tlevn, ipatn)
    ʂ킯郌xƃp^[zŕxw肷. 

    
    * tlevn (Array or NArray) : ʂ킯郌x̒li[z. 
    * ipatn (Array or NArray) : g[p^[ԍi[z.

    l
    (1)̃\bh UESTLN ĂłB
    (2)Tone.stlv Ƃ̊֘AŐ, tlev1 Ƃ tlevn[i], tlev2 Ƃ tlevn[i+1], ipat Ƃ ipatn[i] (i=1,...)  w肵̂Ɠł. , tlevn[i]tlev[i+1] łȂ΂ȂȂ. 킵 DCL manual9.2.5ߎQ.

=end
 
      def itlv
	DCL.ueitlv
      end

# by DCL manual
=begin
---Tone.itlv
    Tone.stlv ܂ Tone.stln Őݒ肵g[x𖳌ɂ.

    
      Ȃ

    l
    (1)̃\bh UEITLV ĂłB
    (2)Tone.stln ܂ Tone.stln ɂĊɐݒ肳Ăg[x𖳌ɂ Tone.draw ĂтƂ, ̃[`ĂԕKv.  

=end

      def levelget
	nlev = DCL.ueqntl
	levels = Array.new
	for i in 1..nlev
	  status = DCL.ueqtlv(i)
	  levels.push(status)
	end
	levels
      end

# by kawanabe
=begin
---Tone.levelget
    ݐݒ肳ĂR^[x̑߂B

    
    Ȃ

    l
    (1) ԂĺA[[tlev1, tlev2, ipat, iton], [tlev1, ... ], ...] Ƃ`ArrayIuWFNgłB

=end

#----------------------------------------------------------
#------------ draw method ----------------------

      def draw(data, x=nil, y=nil, temp=nil)
	if(!(y.is_a?(Hash)))
	  if(x.is_a?(Hash))
	    temp = x.dup
	    x = nil
	  elsif(Basics.typearray(x))
	    if(!Basics.typearray(y))
	      raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	    end
	  end
	else
	  raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	end
	
	nx = data.shape[0]
	ny = data.shape[1]
	
	if(Basics.typearray(x) && Basics.typearray(y))
	  DCL.uwsgxa(x, x.length)
	  DCL.uwsgya(y, y.length)
	  Basics.checkundef(x, y)
	else
	  Basics.checkundef(nx.to_f, ny.to_f)
	end

	@param.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.uetone(data, nx, nx, ny)
	
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Tone.draw(data, x=nil, y=nil, temp=nil)
    g[ɂ2l}̂ʂ킯Ȃ. 

    
    * data (NArray) : f[^ƂȂ2z
    * x, y (Array or NArray) : xWAyW^z
    * temp (Hash) : e|p^

    l
    (1) ̃\bh UETONE ĂłB
    (2) iq_̊ԊusԊȕꍇAƂx, y^ĂƎIɊiq_ݒ肷B
    (3) EBhE̐ݒsĂȂꍇɂāAx, y^΂ƂɃEBhEݒsAx, y^ȂΔz data ̊e̒ƂɃEBhEݒ肷B
    (4) ʂ킯Ȃx̒l Tone.gtla, Tone.gtlb, Tone.stlv ܂ Tone.stln ɂĐݒ肳ĂȂꍇ, ̂悤ɂʂ킯郌xlƃg[p^[ݒ肷. Tone ̊Ǘp^ ltone  true ̂Ƃ, 
          Tone.gtlb(data,-(nlev)) 
        ĂŖ nlev {̃g[xݒ肷Bnlev ̓p^ nlev QƂlł (l12); p^ ltone  false ̂Ƃ, p^ rlev QƂlȉ̗̈p^ ipat QƂl̃g[p^[łʂ킯 (rlev, ipat ̏l͂ꂼ 0.0, 1201:Eオ̎ΐ). Ȃ ltone ̏l false ł. 
    (5) p^ lmiss  true Ȃ猇lȂ. ܂茇l̂܂̊iq_͂ʂ킯Ȃ. (̍, QƂ錇l Fig ̊Ǘ p^ rmiss ł. ) false Ȃ猇lȂȂ. 킵 DCL manual uMATH1ṽ}jAQ. 
    (6) ʂ킯ȂOɂׂĂ̊iq_l̃`FbNȂ. 2̏ꍇʂ킯͂Ȃ킸̎|bZ[Wo͂. 2̏ꍇƂ: 
        * ׂĂ̊iq_l̂Ƃ(missing field). 
        * ׂĂ̊iq_Ȓl̂Ƃ(constant field).

=end
 
      def imdraw(data, x=nil, y=nil, temp=nil)
	if(!(y.is_a?(Hash)))
	  if(x.is_a?(Hash))
	    temp = x.dup
	    x = nil
	  elsif(Basics.typearray(x))
	    if(!Basics.typearray(y))
	      raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	    end
	  end
	else
	  raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	end
	
	nx = data.shape[0]
	ny = data.shape[1]
	
	if(Basics.typearray(x) && Basics.typearray(y))
	  DCL.uwsgxa(x, x.length)
	  DCL.uwsgya(y, y.length)
	  Basics.checkundef(x, y)
	else
	  Basics.checkundef(nx.to_f, ny.to_f)
	end
	
	@param.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.uetonf(data, nx, nx, ny)
	@backup.sync
	@temp_param.clear
      end
      
# by DCL manual
=begin
---Tone.imdraw(data, x=nil, y=nil, temp=nil)
    C[Wf[^`@\g, 2l}̂ʂ킯Ȃ. 

    
    
    * data (NArray) : f[^ƂȂ2z
    * x, y (Array or NArray) : xWAyW^z
    * temp (Hash) : e|p^

    l
    (1) ̃\bh UETONF ĂłB
    (2) iq_̊ԊusԊȕꍇAƂx, y^ĂƎIɊiq_ݒ肷B
    (3) EBhE̐ݒsĂȂꍇɂāAx, y^΂ƂɃEBhEݒsAx, y^ȂΔz data ̊e̒ƂɃEBhEݒ肷B
    (4) Tone.draw Ƃقڌ݊邪, włg[p^[ׂ͂h肾ł. 
    (5) C[W`@\̂ȂfoCXł͎gpłȂ. 
    (6) sNZɎ4_̃f[^`}Ēl (data)  ߐF߂. , F̋E͈ʂɋȐƂȂ.

=end
    
      def fill(upx, upy, temp=nil)
	Basics.checkundef(upx, upy)
	@param.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.sgtnu(upx.length, upx, upy)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Tone.fill(upx, upy, temp=nil)
    p`̈ʂԂ. 

    
    * upx, upy (Array or NArray) : p``钸_UWnɂ(X, Y)W̒l^z.
    * temp (Hash) : e|p^

    l
    (1) ̃\bh SGTNU ĂłB
    (2) upx, upy ̒ 3 ȏłȂ΂ȂȂ. 
    (3) ̈ʂԂg[p^[ԍ tonepat Őݒ肷. 

=end

      def vfill(vpx, vpy, temp=nil)
	Basics.checkundef(vpx, vpy)
	@param.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	DCL.sgtnv(vpx.length, vpx, vpy)
	@backup.sync
	@temp_param.clear
      end
      
# by DCL manual
=begin
---Tone.vfill(vpx, vpy, temp=nil)
    VWnőp`̈ʂԂ. 

    
    * vpx, vpy (Array or NArray) : p``钸_VWnɂ(X, Y)W̒l^z.
    * temp (Hash) : e|p^

    l
    (1) ̃\bh SGTNV ĂłB
    (2) vpx, vpy ̒ 3 ȏłȂ΂ȂȂ. 
    (3) ̈ʂԂg[p^[ԍ tonepat Őݒ肷. 

=end

      def rfill(rpx, rpy, temp=nil)
	Basics.checkundef(rpx, rpy)
	@param.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	DCL.sgtnv(rpx.length, rpx, rpy)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Tone.rfill(rpx, rpy, temp=nil)
    RWnőp`̈ʂԂ. 

    
    * rpx, rpy (Array or NArray) : p``钸_RWnɂ(X, Y)W̒l^z.
    * temp (Hash) : e|p^

    l
    (1) ̃\bh SGTNR ĂłB
    (2) rpx, rpy ̒ 3 ȏłȂ΂ȂȂ. 
    (3) ̈ʂԂg[p^[ԍ tonepat Őݒ肷. 

=end
      module_function :param_init
      module_function :set
      module_function :get
      module_function :draw
      module_function :imdraw
      module_function :gtla
      module_function :gtlb
      module_function :stlv
      module_function :stln
      module_function :itlv
      module_function :levelget
      module_function :fill
      module_function :vfill
      module_function :rfill
      
      param_init
      
#----------------------------------------------------------
    
      def @param.tonepat(itpat)
	DCL.ueiset("itpat", itpat)
	DCL.sgstnp(itpat)
      end
      
      def @temp_param.tonepat(itpat)
	DCL.ueiset("itpat", itpat)
	DCL.sgstnp(itpat)
      end

      def @backup.tonepat(itpat)
	DCL.ueiset("itpat", itpat)
	DCL.sgstnp(itpat)
      end
      
      def @param.tonepatget
	DCL.ueiget("itpat")
      end
      
      @param.keys.each{|i|
	eval "def #{i}; @param['#{i}']; end"
	eval "def #{i}=(val); set('#{i}', val); end"
      }
      
      @param.keys.each{|i|
	eval "module_function :#{i}"
	eval "module_function :#{i}="
      }
    end
  end
end
